#ifdef _BOOL
D_BOOL _BOOL
#endif
#ifdef _EXPLICIT_IS_KEYWORD
D_EXPLICIT_IS_KEYWORD _EXPLICIT_IS_KEYWORD
#endif
#ifdef _LIBC_IN_NAMESPACE_STD_
D_LIBC_IN_NAMESPACE_STD_ _LIBC_IN_NAMESPACE_STD_
#endif
#ifdef _MEMBER_TEMPLATES
D_MEMBER_TEMPLATES _MEMBER_TEMPLATES
#endif
#ifdef _MUTABLE_IS_KEYWORD
D_MUTABLE_IS_KEYWORD _MUTABLE_IS_KEYWORD
#endif
#ifdef _NAMESPACES
D_NAMESPACES _NAMESPACES
#endif
#ifdef _PARTIAL_SPECIALIZATION_OF_CLASS_TEMPLATES
D_PARTIAL_SPECIALIZATION_OF_CLASS_TEMPLATES _PARTIAL_SPECIALIZATION_OF_CLASS_TEMPLATES
#endif
#ifdef _STANDARD_C_PLUS_PLUS
D_STANDARD_C_PLUS_PLUS _STANDARD_C_PLUS_PLUS
#endif
#ifdef _TYPENAME_IS_KEYWORD
D_TYPENAME_IS_KEYWORD _TYPENAME_IS_KEYWORD
#endif
#ifdef _WCHAR_T
D_WCHAR_T _WCHAR_T
#endif
#ifdef _WCHAR_T_IS_KEYWORD
D_WCHAR_T_IS_KEYWORD _WCHAR_T_IS_KEYWORD
#endif
#ifdef __ANSI_CPP__
D__ANSI_CPP__ __ANSI_CPP__
#endif
#ifdef __ARRAY_OPERATORS
D__ARRAY_OPERATORS __ARRAY_OPERATORS
#endif
#ifdef __EDG_ABI_COMPATIBILITY_VERSION
D__EDG_ABI_COMPATIBILITY_VERSION __EDG_ABI_COMPATIBILITY_VERSION
#endif
#ifdef __EDG_RUNTIME_USES_NAMESPACES
D__EDG_RUNTIME_USES_NAMESPACES __EDG_RUNTIME_USES_NAMESPACES
#endif
#ifdef __EDG_VERSION__
D__EDG_VERSION__ __EDG_VERSION__
#endif
#ifdef __EDG__
D__EDG__ __EDG__
#endif
#ifdef __EXCEPTIONS
D__EXCEPTIONS __EXCEPTIONS
#endif
#ifdef __LIBC_MATH_OVERLOAD__
D__LIBC_MATH_OVERLOAD__ __LIBC_MATH_OVERLOAD__
#endif
#ifdef __RTTI
D__RTTI __RTTI
#endif
#ifdef __STDC__
D__STDC__ __STDC__
#endif
