/* Definitions of target machine for GNU compiler, for Blackfin.
   Copyright (C) 2005 Free Software Foundation, Inc.
   Contributed by Analog Devices.

   This file is part of GCC.

   GCC is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 2, or (at your
   option) any later version.

   GCC is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with GCC; see the file COPYING.  If not, write to the
   Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston,
   MA 02110-1301, USA.  */

/* PDImode for the 40 bit accumulators.  */
PARTIAL_INT_MODE (DI);

/* Two of those - covering both accumulators for vector multiplications.  */
VECTOR_MODE (INT, PDI, 2);

VECTOR_MODE (INT, HI, 2); /* V2HI */
VECTOR_MODE (INT, SI, 2); /* V2SI - occasionally used.  */
